@@ -1,6 +1,4 @@
 
-context("process")
-
 test_that("process", {
 
   px <- asNamespace("processx")$get_tool("px")
@@ -27,8 +25,6 @@ test_that("process", {
 })
 
 test_that("process + http", {
-  skip_on_cran()
-  skip("async processes are a bit flaky")
   px <- asNamespace("processx")$get_tool("px")
 
   afun <- async(function() {
